function [res1,res2]=OLS_pull(YY,XX)  
% PURPOSE: computes OLS causality test of data matrices
%---------------------------------------------------
% USAGE:  [res1,res2]  = GLS_pull(YY,XX,nlags) 
% where:            YY = data matrix of dependent (wide form)
%                   XX = data matrix of independent (wide form)
%                nlags = scalar, number of lags in test
%---------------------------------------------------
% RETURNS:
%        res1 = t-statistic of causality
%        res2 = p-value with robust var-cov
% --------------------------------------------------
%
% written by:
% Camacho, Romeu, Ruiz, 2018
% University of Murcia and University of Cartagena

Y=[];
X=[];
for j=1:size(YY,2);
    Yj=YY(:,j);
    Xj=XX(:,j);
    packed = packr([Yj Xj]);
    Yj = packed(:,1);

    Xj = packed(:,2);
    Y=[Y;Yj];
    X=[X;Xj];
end
T=size(Y,1);
vdep=Y(2:T);
vindep=[ones(T-nlags,1) Y(1:T-nlags) X(1:T-nlags)];
beta=inv(vindep'*vindep)*vindep'*vdep;
e=vdep-vindep*beta;
ss=e'*e/(T-size(vindep,2));
varb=ss*inv(vindep'*vindep);
t=beta(3)/sqrt(varb(3,3));
res1=t;
res2=2*(1-normcdf(abs(t)));
end
