% Maximo Camacho, Andres Romeu, Manuel Ruiz 
% www.um.es/econometria/Maximo
% December, 10th 2018 @
% This MATLAB file reads in data from World Development Indicators (2017), World Bank
% Optimization routine for computing a Transfer Entropy Causality Test of Xt->Yt+r;
% Yt->Xt+r and NTE
% Also computes Granger causality Test for comparison

clear;format bank;clc;
rng('default');
%rand('seed',1);
% Select these parameters -----------------------------------------------------
m=3;       % Embedded dimension
r=1;       % lag in the causal effect
B=200;     % Number of bootstrap replications 

% Read and define data --------------------------------------------------------
[TFP,country]=xlsread('data_TFPSize.xlsx',1,'A1:CI29');
[TAM,country]=xlsread('data_TFPSize.xlsx',2,'A1:CI29');
pais=1:size(TFP(1,2:end),2);
anyos = TFP(2:end,1);
Tdim=size(anyos,1); % Time dimension
secciones=kron(pais',ones(Tdim,1));
Npanels=size(pais,2); % Cross-section dimension
NT = Tdim*Npanels;  % Total observations
VarY = TFP(:,2:end);
VarX = TAM(:,2:end);
ntotal = min(numel(VarY(~isnan(VarY))),numel(VarX(~isnan(VarX))));

% OLS Granger estimation ------------------------------------------------------
[GrangerXcY,PGrangerXcY]=GLS_pull(VarY,VarX,r);
[GrangerYcX,PGrangerYcX]=GLS_pull(VarX,VarY,r); 

% HNR estimation --------------------------------------------------------------
Ylong=[];
Xlong=[];
S=[];
[t_i, sects]=hist(secciones,unique(secciones));
for j=1:Npanels; % Pone en formato long
    Yj=VarY(r+1:t_i(j),j);
    Xj=[ VarY(1:t_i(j)-r,j) VarX(1:t_i(j)-r,j)];
    Sj = sects(j)*ones(t_i(j)-r,1);
    Ylong=[Ylong;Yj];
    Xlong=[Xlong;Xj];
    S= [S;Sj];
end
packed = packr([Ylong Xlong S]);

Ydep = packed(:,1);
Xindep = packed(:,2:3);
secciones = packed(:,4);
endogenous=[ 1 0 ];
[bHNR, s2HNR, vcovHNR] = HNR(Ydep,Xindep, secciones, endogenous);
HNRXcY = bHNR(2)/sqrt(vcovHNR(2,2));
pvalHNRXcY = 2*(1-normcdf(abs(HNRXcY)));

Ylong=[];
Xlong=[];
for j=1:Npanels;
    Yj=VarX(r+1:t_i(j),j);
    Xj=[ VarX(1:t_i(j)-r,j) VarY(1:t_i(j)-r,j)];
    Ylong=[Ylong;Yj];
    Xlong=[Xlong;Xj];
end
packed = packr([Ylong Xlong S]);

Ydep = packed(:,1);
Xindep = packed(:,2:3);
secciones = packed(:,4);
[bHNR, s2HNR, vcovHNR] = HNR(Ydep,Xindep, secciones, endogenous);
HNRYcX = bHNR(2)/sqrt(vcovHNR(2,2));
pvalHNRYcX = 2*(1-normcdf(abs(HNRYcX)));

% Dumitrescu-Hurlin and Net transfer entropy ----------------------------------
tot_mhx_XcY=[];tot_mhx_YcX=[];
tot_mhy_XcY=[];tot_mhy_YcX=[];
tot_mhz_XcY=[];tot_mhz_YcX=[];
tot_mhxy_XcY=[];tot_mhxy_YcX=[];
tot_mhxz_XcY=[];tot_mhxz_YcX=[];
tot_mhxyz_XcY=[];tot_mhxyz_YcX=[];
Wj0=0;Wj02=0;

for j=1:Npanels;
  Y=VarY(:,j);
  X=VarX(:,j);
  packed = packr([Y X]);
  Y = packed(:,1);
  X = packed(:,2);
  T=size(Y,1);
  % Estadistico del test DH XcY ---------------------------------------------
  vdep0=Y(r+1:T);
  vindep0=[ones(T-r,1) Y(1:T-r) X(1:T-r)]; % OJO: columna de unos
  beta0=inv(vindep0'*vindep0)*vindep0'*vdep0;
  e0=vdep0-vindep0*beta0;
  varb0=(sum(e0.^2)/(T-r-3))*inv(vindep0'*vindep0); 
  ts0=beta0(3)/sqrt(varb0(3,3));
  Wj0=Wj0+ts0^2; 

  % Estadistico del test DH YcX ---------------------------------------------
  vdep0=X(r+1:T);
  vindep0=[ones(T-r,1) X(1:T-r) Y(1:T-r)]; % OJO: columna de unos
  beta0=inv(vindep0'*vindep0)*vindep0'*vdep0;
  e0=vdep0-vindep0*beta0;
  varb0=(sum(e0.^2)/(T-r-3))*inv(vindep0'*vindep0); 
  ts0=beta0(3)/sqrt(varb0(3,3));
  Wj02=Wj02+ts0^2; 

  % Calculos para el test de NTE --------------------------------------------
  XcY=symbols(Y(1:T-r),Y(r+1:T),X(1:T-r),m);  % X->Y
  YcX=symbols(X(1:T-r),X(r+1:T),Y(1:T-r),m);  % Y->X
  tot_mhx_XcY=[tot_mhx_XcY;XcY.mhx];tot_mhx_YcX=[tot_mhx_YcX;YcX.mhx];
  tot_mhy_XcY=[tot_mhy_XcY;XcY.mhy];tot_mhy_YcX=[tot_mhy_YcX;YcX.mhy];
  tot_mhz_XcY=[tot_mhz_XcY;XcY.mhz];tot_mhz_YcX=[tot_mhz_YcX;YcX.mhz];
  tot_mhxy_XcY=[tot_mhxy_XcY;XcY.mhxy];tot_mhxy_YcX=[tot_mhxy_YcX;YcX.mhxy];
  tot_mhxz_XcY=[tot_mhxz_XcY;XcY.mhxz];tot_mhxz_YcX=[tot_mhxz_YcX;YcX.mhxz];
  tot_mhxyz_XcY=[tot_mhxyz_XcY;XcY.mhxyz];tot_mhxyz_YcX=[tot_mhxyz_YcX;YcX.mhxyz];
  end;
DH_0=sqrt(Npanels/2)*(Wj0/Npanels-1);
DH2_0=sqrt(Npanels/2)*(Wj02/Npanels-1);
EstXcY=TEntropy(tot_mhx_XcY,tot_mhy_XcY,tot_mhz_XcY,tot_mhxy_XcY,tot_mhxz_XcY,tot_mhxyz_XcY);
EstYcX=TEntropy(tot_mhx_YcX,tot_mhy_YcX,tot_mhz_YcX,tot_mhxy_YcX,tot_mhxz_YcX,tot_mhxyz_YcX);
EstNTE=EstXcY.TE-EstYcX.TE;

% p-value by bootstraping -----------------------------------------------------
pvalBSXcY=0;
pvalBSYcX=0;
pvalBSNTE=0;
pvalBSNTE2=0;
pvalDHXcY=0;
pvalDHYcX=0;
kk=waitbar(0,'Bootstrapping. Please wait...');
for Bit=1:B
    waitbar(Bit/B,kk);
    tot_mhx_XcY=[];tot_mhx_YcX=[];
    tot_mhy_XcY=[];tot_mhy_YcX=[];
    tot_mhz_XcY=[];tot_mhz_YcX=[];
    tot_mhxy_XcY=[];tot_mhxy_YcX=[];
    tot_mhxz_XcY=[];tot_mhxz_YcX=[];
    tot_mhxyz_XcY=[];tot_mhxyz_YcX=[];
    Wj=0;Wj2=0;
    for j=1:Npanels;
      Y=VarY(:,j);
      X=VarX(:,j);
      packed = packr([Y X]);
      Y = packed(:,1);
      X = packed(:,2);
      T=size(Y,1);
      % Sacamos la muestra bootstrap ----------------------------------------
      OptBlockY=opt_block(Y);
      OptBlockX=opt_block(X);
      OptBlockY=round(OptBlockY(1,1)+.5);
      OptBlockX=round(OptBlockX(1,1)+.5);      
      [YBoot,YIndB]=stationary_bootstrap(Y,1,OptBlockY);
      [XBoot,XIndB]=stationary_bootstrap(X,1,OptBlockX);

      % Estadistico DH de los Boots XcY -------------------------------------
      vdep=YBoot(r+1:T);
      vindep=[ones(T-r,1) YBoot(1:T-r) XBoot(1:T-r)]; % OJO: columna de unos
      beta=inv(vindep'*vindep)*vindep'*vdep;
      e=vdep-vindep*beta;
      varb=(sum(e.^2)/(T-r-3))*inv(vindep'*vindep); 
      ts=beta(3)/sqrt(varb(3,3));
      Wj=Wj+ts^2; 

      % Estadistico DH de los Boots YcX--------------------------------------
      vdep=XBoot(r+1:T);
      vindep=[ones(T-r,1) XBoot(1:T-r) YBoot(1:T-r)]; % OJO: columna de unos
      beta=inv(vindep'*vindep)*vindep'*vdep;
      e=vdep-vindep*beta;
      varb=(sum(e.^2)/(T-r-3))*inv(vindep'*vindep); 
      ts=beta(3)/sqrt(varb(3,3));
      Wj2=Wj2+ts^2; 

      % Estadistico NTE de los Boots ----------------------------------------
      XcY=symbols(YBoot(1:T-r),YBoot(r+1:T),XBoot(1:T-r),m);  % X->Y
      YcX=symbols(XBoot(1:T-r),XBoot(r+1:T),YBoot(1:T-r),m);  % Y->X
      tot_mhx_XcY=[tot_mhx_XcY;XcY.mhx];tot_mhx_YcX=[tot_mhx_YcX;YcX.mhx];
      tot_mhy_XcY=[tot_mhy_XcY;XcY.mhy];tot_mhy_YcX=[tot_mhy_YcX;YcX.mhy];
      tot_mhz_XcY=[tot_mhz_XcY;XcY.mhz];tot_mhz_YcX=[tot_mhz_YcX;YcX.mhz];
      tot_mhxy_XcY=[tot_mhxy_XcY;XcY.mhxy];tot_mhxy_YcX=[tot_mhxy_YcX;YcX.mhxy];
      tot_mhxz_XcY=[tot_mhxz_XcY;XcY.mhxz];tot_mhxz_YcX=[tot_mhxz_YcX;YcX.mhxz];
      tot_mhxyz_XcY=[tot_mhxyz_XcY;XcY.mhxyz];tot_mhxyz_YcX=[tot_mhxyz_YcX;YcX.mhxyz];
      end;
    DH_boot=sqrt(Npanels/2)*(Wj/Npanels-1);
    DH2_boot=sqrt(Npanels/2)*(Wj2/Npanels-1);
    BootsXcY=TEntropy(tot_mhx_XcY,tot_mhy_XcY,tot_mhz_XcY,tot_mhxy_XcY,tot_mhxz_XcY,...
                      tot_mhxyz_XcY);
    BootsYcX=TEntropy(tot_mhx_YcX,tot_mhy_YcX,tot_mhz_YcX,tot_mhxy_YcX,tot_mhxz_YcX,...
                      tot_mhxyz_YcX);
    BootsNTE=BootsXcY.TE-BootsYcX.TE;
    % P-values del DH ---------------------------------------------------------
    if (abs(DH_boot)>=abs(DH_0))
      pvalDHXcY=pvalDHXcY+(1/B);
      end
    if (abs(DH2_boot)>=abs(DH2_0))
      pvalDHYcX=pvalDHYcX+(1/B);
      end

    % P-values del NTE --------------------------------------------------------
    if (BootsXcY.TE>=EstXcY.TE)
      pvalBSXcY=pvalBSXcY+(1/B);
      end
    if (BootsYcX.TE>=EstYcX.TE)
      pvalBSYcX=pvalBSYcX+(1/B);
      end
    if (EstNTE>=0)
      if (BootsNTE>=EstNTE)
        pvalBSNTE=pvalBSNTE+(1/B);
        end
        else          
        if (BootsNTE<EstNTE)
          pvalBSNTE=pvalBSNTE+(1/B);
          end
        end
    if (abs(BootsNTE)>abs(EstNTE))
      pvalBSNTE2=pvalBSNTE2+(1/B); 
      end  
  end
close(kk); 

% Output results --------------------------------------------------------------
fprintf('===========================================================================\n');
fprintf(' TRANSFER ENTROPY CAUSALITY TEST \n');
fprintf(' Tests FIRM SIZE -> TFP, TFP -> FIRM SIZE and NTE\n');
fprintf(' Data from the BLS on %d countries and %d years. %d observations\n',Npanels,Tdim,ntotal);
fprintf(' Number of lags: %d  Number of symbols: %d\n',r,m);
fprintf('---------------------------------------------------------------------------\n');
fprintf(' Direction of                                                              \n');
fprintf(' causality      GLS-test    Holtz-Eakin     DHurlin      Transfer Entropy  \n');
fprintf('              ------------ ------------- ------------- --------------------\n');
fprintf('                                                                 P-Value   \n');
fprintf('               Test  P-Val   Test  P-Val   Test  P-Val   Test  1tail  2side\n');
fprintf('---------------------------------------------------------------------------\n');
fprintf(' X -> Y     %7.3f%7.3f%7.3f%7.3f%7.3f%7.3f%7.3f%7.3f\n',...
            GrangerXcY,PGrangerXcY,HNRXcY,pvalHNRXcY,DH_0,pvalDHXcY,EstXcY.TE,pvalBSXcY);
fprintf(' Y -> X     %7.3f%7.3f%7.3f%7.3f%7.3f%7.3f%7.3f%7.3f\n',...
            GrangerYcX,PGrangerYcX,HNRYcX,pvalHNRYcX,DH2_0,pvalDHYcX,EstYcX.TE,pvalBSYcX);
fprintf(' Net        %49.3f%7.3f%7.3f\n',EstNTE,pvalBSNTE,pvalBSNTE2);
fprintf('---------------------------------------------------------------------------\n');


